--
-- c_gui.lua
--

-----------------------------------------------------
--
-- Key:      num_8     - Toggles GUI
-- Command: 'flagdump' - Dumps slider values to console and clipboard
--
-----------------------------------------------------

useGUI = true		-- Set to true to enable GUI
scx, scy = guiGetScreenSize ()

Settings = {}
Settings.var = {}

--------------------------------------
if useGUI then

	Inputs = {}
	local mainWindow

	flo = 0
	fhi = 100

	bindKey( "num_8", "down",
		function()
			if guiGetVisible(mainWindow) then
				guiSetVisible(mainWindow, false)
				showCursor(false)
			else
				guiSetVisible(mainWindow, true)
				showCursor(true)
			end
		end
	)


	addEventHandler("onClientResourceStop",resourceRoot,
		function()
			saveSettings()
		end
	)



	function initSliders( title, initTableData )
		-- Create gui
		mainWindow = guiCreateWindow(129,235,527,2095,title,false)
		local ypos = 30
		local yposmax = 30
		local xpos = 0

		-- Init data
		for _,row in ipairs(initTableData) do
			if type(row[1]) == "string" then
				ypos = ypos - 1
				guiCreateLabel(xpos+39,ypos,132,17,row[1],false,mainWindow)
				ypos = ypos + 15
			else
				local name = row[2]
				Inputs[name] = { showPrecision=row[1], name=name, tlo=row[3], thi=row[4], default=row[5] }

				Inputs[name].scrollbar = guiCreateScrollBar(xpos+39,ypos,160,17,true,false,mainWindow)
				Inputs[name].valueLabel = guiCreateLabel(xpos+209,ypos,26,19,"0",false,mainWindow)
				Inputs[name].label = guiCreateLabel(xpos+239,ypos,112,17,name,false,mainWindow)
				ypos = ypos + 22
			end

			yposmax = math.max(yposmax,ypos)
			if ypos + 100 > scy then
				ypos = 30
				xpos = xpos + 330
			end
		end

		-- Adjust window size
		local sx,sy = guiGetSize( mainWindow, false )
		sy = yposmax + 20
		sx = xpos+239 + 112 + 39
		guiSetSize( mainWindow, sx, sy, false )

		showCursor(true)
		loadSettings()
	end

	----------------------------------------------------------------
	-- Save & load
	----------------------------------------------------------------
	-- saveToFile
	function saveToFile()
		local settingsFile = xmlLoadFile( "settings.xml" )
		if not settingsFile then
			settingsFile = xmlCreateFile( "settings.xml", "settings" )
			if not settingsFile then return false end
		end

		local node = xmlFindChild( settingsFile, "data", 0 )
		if not node then
			node = xmlCreateChild( settingsFile, "data" )
		end
		local asString = toJSON( Settings )
		--	outputDebugString( "saveToFile "
		--		.. " asString:" .. tostring(asString)
		--		)
		xmlNodeSetValue( node, asString )
		xmlSaveFile( settingsFile )
		xmlUnloadFile( settingsFile )
	end

	-- loadFromFile
	function loadFromFile()
		local settingsFile = xmlLoadFile( "settings.xml" )
		if not settingsFile then
			return
		end

		local node = xmlFindChild( settingsFile, "data", 0 )
		if node then
			local asString = xmlNodeGetValue( node )
			--outputDebugString( "loadFromFile"
			--	.. " asString:" .. tostring(asString)
			--	)
			Settings = fromJSON( asString )
		end
		xmlUnloadFile( settingsFile )
	end

	-- saveSettings
	function saveSettings()
		if not Settings.var then return end
		if not mainWindow then return end

		for name,value in pairs(Settings.var) do
			if not Inputs[name] then
				outputDebugString ( "in var, but not Inputs"
					.. " name:" .. tostring(name)
					.. " value:" .. tostring(value)
					)
				Settings.var[name] = nil
			end
		end
		for name,value in pairs(Settings.gui) do
			if not Inputs[name] then
				outputDebugString ( "in gui, but not Inputs"
					.. " name:" .. tostring(name)
					.. " value:" .. tostring(value)
					)
				Settings.gui[name] = nil
			end
		end

		Settings.window = {}
		Settings.window.position = {guiGetPosition( mainWindow, false )}
		saveToFile()
	end

	-- loadSettings
	function loadSettings()
		loadFromFile ()
		validateSettings_gui_to_var()
		if Settings.window and Settings.window.position and mainWindow then
			guiSetPosition( mainWindow, Settings.window.position[1], Settings.window.position[2], false )
		end
		updateGUIFromSettings_gui()
		updateSettingsFromGUI()
	end

	-- saveSettings
	function varMap_gui_to_var( tab, name, tlo, thi, default, bInt )
		local fvalue = tonumber(tab.gui[name])
		if not fvalue then
			local pos = math.unlerpclamped( tlo, default, thi )
			fvalue = math.lerp( flo, pos, fhi )
		end
		local pos = math.unlerpclamped( flo, fvalue, fhi )
		local tvalue = math.lerp( tlo, pos, thi )
		tab.gui[name] = fvalue
		if bInt then
			tvalue = math.floor(tvalue+.5)
		end
		tab.var[name] = tvalue
	end

	function varMap_var_to_gui( tab, name, tlo, thi, default, bInt )
		local tvalue = tonumber(tab.var[name])
		if not tvalue then
			tvalue = math.clamp( tlo, default, thi )
		end
		local pos = math.unlerpclamped( tlo, tvalue, thi )
		local fvalue = math.lerp( flo, pos, fhi )
		tab.gui[name] = fvalue
		if bInt then
			tvalue = math.floor(tvalue+.5)
		end
		tab.var[name] = tvalue
	end


	-- saveSettings
	function validateSettings_gui_to_var()
		if not Settings     then Settings     = {} end
		if not Settings.gui then Settings.gui = {} end
		if not Settings.var then Settings.var = {} end
		for name,info in pairs(Inputs) do
			varMap_gui_to_var( Settings, info.name, info.tlo, info.thi, info.default, info.showPrecision==0 )
		end
	end

	function validateSettings_var_to_gui()
		if not Settings     then Settings     = {} end
		if not Settings.gui then Settings.gui = {} end
		if not Settings.var then Settings.var = {} end
		for name,info in pairs(Inputs) do
			varMap_var_to_gui( Settings, info.name, info.tlo, info.thi, info.default, info.showPrecision==0 )
		end
	end

	-- saveSettings
	function updateGUIFromSettings_gui()
		removeEventHandler( "onClientGUIScroll", resourceRoot, updateSettingsFromGUI )
		for name,info in pairs(Inputs) do
			guiScrollBarSetScrollPosition( info.scrollbar, Settings.gui[info.name] )
		end
		addEventHandler( "onClientGUIScroll", resourceRoot, updateSettingsFromGUI )
	end

	-- updateSettingsFromGUI
	function updateSettingsFromGUI()
		for name,info in pairs(Inputs) do
			Settings.gui[info.name] = guiScrollBarGetScrollPosition( info.scrollbar )
		end
		validateSettings_gui_to_var()
		if saveTimer then
			killTimer(saveTimer)
		end
		saveTimer = setTimer( function () saveTimer = nil saveSettings() end , 500, 1 )

		for name,info in pairs(Inputs) do
			guiSetText( info.valueLabel, formatPrecision(info.showPrecision,Settings.var[info.name]) )
		end
		triggerEvent( "onClientChangedSettings", resourceRoot )
	end


	----------------------------------------------------------------
	-- dumpSettings
	function dumpValues(key, state )
		local output = ""
		output = output .. "---------------------------------\n"
		output = output .. "function setTypeX()\n"
		output = output .. "    local v = Settings.var\n"
		for name,value in pairs(Settings.var) do
			local v = tostring(value)
			if string.len(v) > 5 then
				v = string.format( "%1.2f", value )
			end
			output = output .. "    v." .. tostring(name) .. "=" .. v .. "\n"
		end
		output = output .. "end\n"
		output = output .. "---------------------------------\n"
		outputConsole( output )
		setClipboard ( output )
	end
	addCommandHandler ( "flagdump", dumpValues )

end


----------------------------------------------------------------
-- Mafs
----------------------------------------------------------------
function formatPrecision(p,value)
	local str = string.format( "%%.%df", p )
	return string.format( str, value )
end

function math.lerp(from,alpha,to)
    return from + (to-from) * alpha
end

function math.unlerp(from,pos,to)
	if ( to == from ) then
		return 1
	end
	return ( pos - from ) / ( to - from )
end


function math.clamp(low,value,high)
    return math.max(low,math.min(value,high))
end

function math.unlerpclamped(from,pos,to)
	return math.clamp(0,math.unlerp(from,pos,to),1)
end
